﻿using System;
using System.Collections.Generic;
using System.Linq;
using Interfaces;

namespace HR
{
    public class HrManager : IHr
    {
        class _Employee
        {
            public string EmpNo { get; set; }
            public string Name { get; set; }
            public DateTime Birthday { get; set; }
            public string Descriptions { get; set; }
            public EmployeeStatus Status { get; set; }
            public decimal Salary { get; set; }
        }

        private const string EMP_DOES_NOT_EXIST = "员工不存在";
        private const string EMP_ALREADY_EXISTS = "员工已存在";
        private const string ONLY_PROBATION = "只有试用期员工可转正";
        private const string ONLY_NORMAL = "只有正式员工可以加薪";

        private static readonly List<_Employee> _data = new List<_Employee>
        {
            new _Employee{EmpNo = "0001", Name = "蒋国纲", Birthday = new DateTime(1981, 11, 12), Descriptions = "技术宅男", Status=EmployeeStatus.Normal, Salary=5000.00m},
            new _Employee{EmpNo = "0002", Name = "周星驰", Birthday = new DateTime(1962, 6, 22), Descriptions = "喜剧之王", Status=EmployeeStatus.Probation, Salary = 2000.00m},
            new _Employee{EmpNo = "0003", Name = "李逍遥", Birthday = new DateTime(1958, 12, 19), Descriptions = "蜀山派掌门人", Status=EmployeeStatus.Leaved, Salary = 8000.00m}
        };

        public IEnumerable<Employee_Get> GetAllEmployees()
        {
            //这个其实可以用自动映射来简化代码，以下同理
            return _data.Select( e => new Employee_Get
                        {
                            EmpNo = e.EmpNo,
                            Name = e.Name,
                            Birthday = e.Birthday,
                            Descriptions = e.Descriptions,
                            Status = e.Status,
                            Salary = e.Salary
                        });
        }

        public Employee_Get GetEmployee(string empno)
        {
            _Employee emp = _data.SingleOrDefault(e => e.EmpNo == empno);
            if (emp==null)
            {
                throw new DemoException(EMP_DOES_NOT_EXIST);
            }
            return new Employee_Get
                       {
                           EmpNo = emp.EmpNo,
                           Name = emp.Name,
                           Birthday = emp.Birthday,
                           Descriptions = emp.Descriptions,
                           Status = emp.Status,
                           Salary = emp.Salary
                       };
        }

        public void AddEmployee(Employee_Add emp)
        {
            if (_data.Exists(e => e.EmpNo == emp.EmpNo))
            {
                throw new DemoException(EMP_ALREADY_EXISTS);
            }
            _data.Add(new _Employee
                          {
                              EmpNo = emp.EmpNo,
                              Name = emp.Name,
                              Birthday = emp.Birthday,
                              Descriptions = emp.Descriptions,
                              Status = EmployeeStatus.Probation,
                              Salary = 2000.00m //假设试用期默认薪资2000
                          });
        }

        public void EditEmployee(string empno, Employee_Edit emp)
        {
            _Employee empData = _data.SingleOrDefault(e => e.EmpNo == empno);
            if (empData == null)
            {
                throw new DemoException(EMP_DOES_NOT_EXIST);
            }
            empData.Name = emp.Name;
            empData.Birthday = emp.Birthday;
            empData.Descriptions = emp.Descriptions;
        }

        public void DeleteEmployee(string empno)
        {
            _Employee empData = _data.SingleOrDefault(e => e.EmpNo == empno);
            if (empData == null)
            {
                throw new DemoException(EMP_DOES_NOT_EXIST);
            }
            _data.Remove(empData);
        }

        public void BecomeRegular(string empno)
        {
            _Employee empData = _data.SingleOrDefault(e => e.EmpNo == empno);
            if (empData == null)
            {
                throw new DemoException(EMP_DOES_NOT_EXIST);
            }
            if (empData.Status!=EmployeeStatus.Probation)
            {
                throw new DemoException(ONLY_PROBATION);
            }
            empData.Status = EmployeeStatus.Normal;
        }

        public void BecomeLeave(string empno)
        {
            _Employee empData = _data.SingleOrDefault(e => e.EmpNo == empno);
            if (empData == null)
            {
                throw new DemoException(EMP_DOES_NOT_EXIST);
            }
            empData.Status = EmployeeStatus.Leaved;
        }

        public void IncreaseSalary(string empno, decimal scope)
        {
            _Employee empData = _data.SingleOrDefault(e => e.EmpNo == empno);
            if (empData == null)
            {
                throw new DemoException(EMP_DOES_NOT_EXIST);
            }
            if (empData.Status != EmployeeStatus.Normal)
            {
                throw new DemoException(ONLY_NORMAL);
            }
            empData.Salary += scope;
        }

        public bool IsEmployeeExist(string empno)
        {
            return _data.Exists(e => e.EmpNo == empno);
        }
    }
}
